'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var Plugin = _ref.Plugin;
  var t = _ref.types;

  return {
    visitor: {
      Program: function Program(path) {
        // On program start, do an explicit traversal up front for this plugin.
        path.traverse({
          ObjectProperty: {
            exit: function exit(path) {
              var node = path.node;

              if (node.computed || node.key.name !== 'propTypes') {
                return;
              }

              var parent = path.findParent(function (parent) {
                if (parent.type !== 'CallExpression') {
                  return false;
                }

                return parent.get('callee').matchesPattern('React.createClass');
              });

              if (parent) {
                path.remove();
              }
            }
          },
          ClassProperty: function ClassProperty(path) {
            var node = path.node;
            var scope = path.scope;


            if (node.key.name === 'propTypes') {
              var className = scope.block.id.name;
              var binding = scope.getBinding(className);
              var superClass = binding.path.get('superClass');

              if (superClass.matchesPattern('React.Component') || superClass.node.name === 'Component') {
                path.remove();
              } else if (superClass.node.name) {
                // Check for inheritance
                className = superClass.node.name;
                binding = scope.getBinding(className);
                superClass = binding.path.get('superClass');

                if (superClass.matchesPattern('React.Component') || superClass.node && superClass.node.name === 'Component') {
                  path.remove();
                }
              }
            }
          },
          AssignmentExpression: function AssignmentExpression(path) {
            var node = path.node;
            var scope = path.scope;


            if (node.left.computed || !node.left.property || node.left.property.name !== 'propTypes') {
              return;
            }

            var className = node.left.object.name;
            var binding = scope.getBinding(className);

            if (!binding) {
              return;
            }

            if (binding.path.isClassDeclaration()) {
              var superClass = binding.path.get('superClass');
              if (superClass.matchesPattern('React.Component') || superClass.matchesPattern('Component')) {
                path.remove();
              }
            } else if ((0, _isStatelessComponent2.default)(binding.path)) {
              path.remove();
            }
          }
        });
      }
    }
  };
};

var _isStatelessComponent = require('./isStatelessComponent');

var _isStatelessComponent2 = _interopRequireDefault(_isStatelessComponent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }